@@ -1,9 +1,8 @@
 use std::{ptr, str, slice};
 use std::marker::PhantomData;
-use std::ops::{Deref};
+use std::ops::{Deref, Index};
 use std::alloc::{self,Layout,System,GlobalAlloc};
-use std::ffi::{c_void, CStr};
-use std::os::raw::c_char;
+use std::ffi::{c_void};
 use std::cmp::min;
 use std::io::{Read,Seek,SeekFrom};
 use std::fs::File;
@@ -22,6 +21,12 @@ pub struct List<T> {
     _marker: PhantomData<T>,
 }
 
+impl<T> List<T> {
+    pub(crate) unsafe fn as_static_ref(&self) -> &'static [T] {
+        slice::from_raw_parts(self.data, self.count)
+    }
+}
+
 impl<T> AsRef<[T]> for List<T> {
     fn as_ref(&self) -> &[T] {
         unsafe { slice::from_raw_parts(self.data, self.count) }
@@ -43,6 +48,13 @@ impl<'a, T> IntoIterator for &'a List<T> {
     }
 }
 
+impl<T> Index<usize> for List<T> {
+    type Output = T;
+    fn index(&self, index: usize) -> &T {
+        &self.as_ref()[index]
+    }
+}
+
 #[repr(C)]
 pub struct RefList<T> {
     data: *const Ref<T>,
@@ -50,6 +62,12 @@ pub struct RefList<T> {
     _marker: PhantomData<T>,
 }
 
+impl<T> RefList<T> {
+    pub(crate) unsafe fn as_static_ref(&self) -> &'static [Ref<T>] {
+        slice::from_raw_parts(self.data, self.count)
+    }
+}
+
 impl<T> AsRef<[Ref<T>]> for RefList<T> {
     fn as_ref(&self) -> &[Ref<T>] {
         unsafe { slice::from_raw_parts(self.data, self.count) }
@@ -63,11 +81,29 @@ impl<T> Deref for RefList<T> {
     }
 }
 
+pub struct RefIter<'a, T> {
+    inner: slice::Iter<'a, Ref<T>>,
+}
+
+impl<'a, T> Iterator for RefIter<'a, T> {
+    type Item = &'a T;
+    fn next(&mut self) -> Option<Self::Item> {
+        self.inner.next().map(|v| v.as_ref())
+    }
+}
+
 impl<'a, T> IntoIterator for &'a RefList<T> {
-    type Item = &'a Ref<T>;
-    type IntoIter = slice::Iter<'a, Ref<T>>;
-    fn into_iter(self) -> Self::IntoIter {
-        self.as_ref().into_iter()
+    type Item = &'a T;
+    type IntoIter = RefIter<'a, T>;
+    fn into_iter(self) -> RefIter<'a, T> {
+        RefIter::<'_, T> { inner: self.as_ref().into_iter() }
+    }
+}
+
+impl<T> Index<usize> for RefList<T> {
+    type Output = T;
+    fn index(&self, index: usize) -> &T {
+        &self.as_ref()[index]
     }
 }
 
@@ -77,6 +113,12 @@ pub struct Ref<T> {
     _marker: PhantomData<T>,
 }
 
+impl<T> AsRef<T> for Ref<T> {
+    fn as_ref(&self) -> &T {
+        unsafe { &*self.ptr.as_ptr() }
+    }
+}
+
 impl<T> Deref for Ref<T> {
     type Target = T;
     fn deref(&self) -> &Self::Target {
@@ -131,6 +173,12 @@ pub struct String {
     _marker: PhantomData<u8>,
 }
 
+impl String {
+    pub(crate) unsafe fn as_static_ref(&self) -> &'static str {
+        str::from_utf8_unchecked(slice::from_raw_parts(self.data, self.length))
+    }
+}
+
 impl AsRef<str> for String {
     fn as_ref(&self) -> &str {
         unsafe { str::from_utf8_unchecked(slice::from_raw_parts(self.data, self.length)) }
@@ -465,22 +513,34 @@ impl Debug for Error {
     }
 }
 
-#[link(name="ufbx")]
-extern "C" {
-    pub fn ufbxi_rust_pop_assert() -> *const c_char;
+#[repr(C)]
+pub struct ExternalRef<'a, T> {
+    data: T,
+    _marker: PhantomData<&'a T>,
 }
 
-pub fn assert_to_panic() {
-    unsafe {
-        let pointer = ufbxi_rust_pop_assert();
-        if !pointer.is_null() {
-            let c_message = CStr::from_ptr(pointer);
-            let message = c_message.to_str().unwrap_or("(bad assert message)");
-            panic!("{}", message);
+impl<'a, T> ExternalRef<'a, T> {
+    pub unsafe fn new(t: T) -> Self {
+        Self {
+            data: t,
+            _marker: PhantomData,
         }
     }
 }
 
+impl<'a, T> AsRef<T> for ExternalRef<'a, T> {
+    fn as_ref(&self) -> &T {
+        &self.data
+    }
+}
+
+impl<'a, T> Deref for ExternalRef<'a, T> {
+    type Target = T;
+    fn deref(&self) -> &Self::Target {
+        &self.data
+    }
+}
+
 /*
 
 #[repr(C)]
